@@ -5,8 +5,13 @@ import json
 
 def test_add(api):
     config = api.prefix_config()
-    assert config.a == config._DEFAULTS["a"]
-    assert config.b == config._DEFAULTS["b"]
+    config.a = "asdf"
+    config.b = 1.1
+    config.c = 1
+    config.required_object.e_a = 1.1
+    config.required_object.e_b = 1.2
+    assert config.f.f_a == config.f._DEFAULTS["f_a"]
+    assert config.f.f_b == config.f._DEFAULTS["f_b"]
     g1 = config.g.add(name="unique list name", g_a="dkdkd", g_b=3, g_c=22.2)
     g1.g_d = "gdgdgd"
     j = config.j.add()
